@@ -68,6 +68,8 @@ pub type SInt = i32;
 
 /// Integer coordinate type.
 pub type Coord = i32;
+/// Integer area type.
+pub type Area = i64;
 
 /// Circuit identifier.
 #[derive(Debug, Copy, Clone, Eq, PartialEq, Hash)]
@@ -122,6 +124,24 @@ pub type ShapeId = Index<Shape<Coord>, u32>;
 /// ID for layers.
 pub type LayerId = Index<LayerInfo<NameT>, u16>;
 
+/// Allow creating IDs from integers.
+macro_rules! impl_from_for_id {
+    ($t:tt, $i:ty) => {
+        impl From<$i> for $t {
+            fn from(id: $i) -> Self {
+                $t(id)
+            }
+        }
+    }
+}
+
+impl_from_for_id!(CellId, u32);
+impl_from_for_id!(CellInstId, usize);
+impl_from_for_id!(PinId, u32);
+impl_from_for_id!(PinInstId, usize);
+impl_from_for_id!(NetId, usize);
+
+
 /// A circuit is defined by an interface (pins) and
 /// a content which consists of interconnected circuit instances.
 ///
@@ -144,8 +164,6 @@ pub struct Circuit<C = Coord, U = ()>
     instances_by_name: HashMap<NameT, CellInstId>,
     /// Circuit instances that reference to this circuit.
     references: IntHashSet<CellInstId>,
-    /// All circuits that have instances of this circuit.
-    parents: IntHashSet<CellId>,
 
     /// Set of circuits that are direct dependencies of this circuit.
     /// Stored together with a counter of how many instances of the dependency are present.
@@ -206,7 +224,7 @@ impl Circuit {
     fn get_or_create_shapes_mut(&mut self, layer_id: &LayerId) -> &mut Shapes<Coord> {
         let self_id = self.id();
         self.shapes_map.entry(*layer_id)
-            .or_insert_with(|| Shapes::new(self_id))
+            .or_insert_with(|| Shapes::new())
             .borrow_mut()
     }
 }
@@ -268,8 +286,6 @@ impl CircuitInst {
 #[derive(Debug, Clone)]
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
 pub struct Pin {
-    /// The unique ID of the pin.
-    id: PinId,
     /// Name of the pin.
     name: NameT,
     /// Signal type/direction of the pin.
@@ -278,8 +294,6 @@ pub struct Pin {
     circuit: CellId,
     /// Net that is connected to this pin.
     net: Option<NetId>,
-    /// Position in the list of pins of the parent circuit.
-    position: usize,
 
     // == Layout == //
     /// List of shapes in the layout that represent the physical pin.
@@ -449,7 +463,6 @@ impl Chip<Coord> {
             instances: Default::default(),
             instances_by_name: Default::default(),
             references: Default::default(),
-            parents: Default::default(),
             nets: Default::default(),
             nets_by_name: Default::default(),
             // Create LOW and HIGH nets.
@@ -517,8 +530,8 @@ impl Chip<Coord> {
 
     /// Create a new instance of `circuit_template` in the `parent` circuit.
     fn create_circuit_instance(&mut self, parent: &CellId,
-                                   circuit_template: &CellId,
-                                   name: Option<NameT>) -> CellInstId {
+                               circuit_template: &CellId,
+                               name: Option<NameT>) -> CellInstId {
         let id = CellInstId(Self::next_id_counter_usize(&mut self.id_counter_circuit_inst));
 
         {
@@ -857,14 +870,11 @@ impl Chip<Coord> {
     /// Update all circuit instances with the new pin.
     fn create_pin(&mut self, parent: CellId, name: NameT, direction: Direction) -> PinId {
         let pin_id = PinId(Self::next_id_counter_u32(&mut self.id_counter_pin));
-        let position = self.num_pins(&parent);
         let pin = Pin {
             name,
             direction,
             circuit: parent,
             net: Default::default(),
-            id: pin_id,
-            position,
             pin_shapes: Default::default(),
         };
         self.pins.insert(pin_id, pin);
@@ -1188,10 +1198,6 @@ pub struct Shape<C, U = ()> {
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
 pub struct Shapes<C>
     where C: CoordinateType {
-    // /// ID of this shape collection.
-    // id: Index<Self>,
-    /// Reference to the cell where this shape collection lives. Can be none.
-    parent_cell: CellId,
     /// Shape elements.
     shapes: IntHashMap<ShapeId, Shape<C>>,
     /// Property stores for the shapes.
@@ -1200,10 +1206,8 @@ pub struct Shapes<C>
 
 impl<C: CoordinateType> Shapes<C> {
     /// Create a new empty shapes container.
-    fn new(parent_id: CellId) -> Self {
+    fn new() -> Self {
         Self {
-            // id,
-            parent_cell: parent_id,
             shapes: Default::default(),
             shape_properties: Default::default(),
         }
@@ -1350,6 +1354,7 @@ impl HierarchyBase for Chip<Coord> {
 
 impl LayoutBase for Chip<Coord> {
     type Coord = Coord;
+    type Area = Area;
     type LayerId = LayerId;
     type ShapeId = ShapeId;
 
@@ -1361,8 +1366,8 @@ impl LayoutBase for Chip<Coord> {
         Box::new(self.layer_info.keys().copied())
     }
 
-    fn layer_info(&self, layer: &Self::LayerId) -> &LayerInfo<Self::NameType> {
-        &self.layer_info[layer]
+    fn layer_info(&self, layer: &Self::LayerId) -> LayerInfo<Self::NameType> {
+        self.layer_info[layer].clone()
     }
 
     fn find_layer(&self, index: u32, datatype: u32) -> Option<Self::LayerId> {
@@ -1433,7 +1438,7 @@ impl LayoutBase for Chip<Coord> {
         self.circuit_inst(cell_inst).get_transform().clone()
     }
 
-    fn get_shape_property(&mut self, shape: &Self::ShapeId, key: &Self::NameType) -> Option<PropertyValue> {
+    fn get_shape_property(&self, shape: &Self::ShapeId, key: &Self::NameType) -> Option<PropertyValue> {
         let (cell, layer) = self.shape_parents[shape].clone();
         self.circuit(&cell)
             .shapes_map[&layer]
