@@ -18,10 +18,14 @@
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
-//! Wrapper around netlist, layout and L2N structures that allows undoing of operations.
-//!
+//! *Experimental*: Wrapper around netlist, layout and L2N structures that allows undoing of operations.
 //!
 //! This is work in progress.
+//! Missing things are:
+//! * Undoing a operation on the hierarchy does not necessarily restore netlist, layout and l2n information.
+//! * Undoing a netlist operation does not restore l2n information.
+//! * Undoing a layout operation does not restore l2n information.
+//! * Undoing does not restore user-defined properties.
 //!
 //! # Caveat
 //! Undoing removal of some objects does not preserve the ID of the object.
@@ -30,9 +34,12 @@
 
 use crate::traits::*;
 use crate::netlist::direction::Direction;
-use crate::layout::prelude::{Rect, LayerInfo, Geometry, SimpleTransform};
+use crate::layout::prelude::{Geometry, SimpleTransform};
 use std::ops::Deref;
 use crate::prelude::PropertyValue;
+use crate::decorator::hierarchy::HierarchyBaseDecorator;
+use crate::decorator::{Decorator, MutDecorator};
+use crate::decorator::layout::LayoutBaseDecorator;
 
 /// Undo operations on the netlist.
 pub enum NetlistUndoOp<T: NetlistBase> {
@@ -82,7 +89,7 @@ pub enum LayoutUndoOp<T: LayoutBase> {
         geometry: Geometry<T::Coord>,
     },
     /// Store the old geometry of the shape.
-    ReplaceShape (T::ShapeId, Geometry<T::Coord>),
+    ReplaceShape(T::ShapeId, Geometry<T::Coord>),
     /// Store the old transform.
     SetTransform(T::CellInstId, SimpleTransform<T::Coord>),
 
@@ -95,10 +102,24 @@ impl<T: LayoutBase> From<HierarchyUndoOp<T>> for LayoutUndoOp<T> {
 }
 
 /// Undo operation for `L2NEdit` operations.
-enum L2NUndoOp<T: L2NBase> {
+#[allow(missing_docs)]
+pub enum L2NUndoOp<T: L2NBase> {
+    /// Undo an operation on the cell hierarchy.
     HierarchyOp(HierarchyUndoOp<T>),
+    /// Undo a netlist operation.
     NetlistOp(NetlistUndoOp<T>),
+    /// Undo a layout operation.
     LayoutOp(LayoutUndoOp<T>),
+    /// Undo setting the net of a shape.
+    SetNetOfShape {
+        shape_id: T::ShapeId,
+        previous_net: Option<T::NetId>,
+    },
+    /// Undo setting the pin of a shape.
+    SetPinOfShape {
+        shape_id: T::ShapeId,
+        previous_pin: Option<T::PinId>,
+    },
 }
 
 impl<T: L2NBase> From<HierarchyUndoOp<T>> for L2NUndoOp<T> {
@@ -126,17 +147,31 @@ pub enum HierarchyUndoOp<T: HierarchyBase> {
     /// Undo creating a cell instance.
     CreateCellInstance(T::CellInstId),
     /// Holds the previous name of the cell.
-    RenameCell(T::CellId, T::NameType),
+    RenameCell {
+        /// The renamed cell.
+        cell: T::CellId,
+        /// The name to be restored when undoing.
+        previous_name: T::NameType,
+    },
     /// Holds the previous name of the cell instance.
-    RenameCellInst(T::CellInstId, Option<T::NameType>),
+    RenameCellInst {
+        /// The renamed instance.
+        inst: T::CellInstId,
+        /// The name to be restored when undoing.
+        previous_name: Option<T::NameType>,
+    },
 }
 
 /// Wrapper around netlist, layout and L2N structures that allows undoing of operations.
+///
+/// # Types
+/// * `T`: Underlying data structure.
+/// * `U`: Undo-operation.
 pub struct Undo<'a, T, U> {
     /// Underlying data structure.
     chip: &'a mut T,
     /// A list of performed transactions.
-    /// To undo operations, the list has to be worked through from the end.
+    /// To undo operations, this list has to be worked through from the end.
     transactions: Vec<U>,
 }
 
@@ -168,15 +203,33 @@ impl<'a, T: HierarchyEdit, U> Undo<'a, T, U> {
                 self.chip.remove_cell(&c),
             HierarchyUndoOp::CreateCellInstance(c) =>
                 self.chip.remove_cell_instance(&c),
-            HierarchyUndoOp::RenameCell(c, n) =>
-                self.chip.rename_cell(&c, n),
-            HierarchyUndoOp::RenameCellInst(c, n) =>
-                self.chip.rename_cell_instance(&c, n)
+            HierarchyUndoOp::RenameCell { cell, previous_name } =>
+                self.chip.rename_cell(&cell, previous_name),
+            HierarchyUndoOp::RenameCellInst { inst, previous_name } =>
+                self.chip.rename_cell_instance(&inst, previous_name)
         }
     }
 }
 
 
+impl<'a, T: L2NBase + 'static, U> L2NBase for Undo<'a, T, U> {
+    fn shapes_of_net(&self, net_id: &Self::NetId) -> Box<dyn Iterator<Item=Self::ShapeId> + '_> {
+        self.chip.shapes_of_net(net_id)
+    }
+
+    fn shapes_of_pin(&self, pin_id: &Self::PinId) -> Box<dyn Iterator<Item=Self::ShapeId> + '_> {
+        self.chip.shapes_of_pin(pin_id)
+    }
+
+    fn get_net_of_shape(&self, shape_id: &Self::ShapeId) -> Option<Self::NetId> {
+        self.chip.get_net_of_shape(shape_id)
+    }
+
+    fn get_pin_of_shape(&self, shape_id: &Self::ShapeId) -> Option<Self::PinId> {
+        self.chip.get_pin_of_shape(shape_id)
+    }
+}
+
 impl<'a, T: L2NEdit, U> Undo<'a, T, U> {
     /// Undo an operation on fused netlist and layout.
     fn undo_l2n_op(&mut self, op: L2NUndoOp<T>) {
@@ -186,6 +239,18 @@ impl<'a, T: L2NEdit, U> Undo<'a, T, U> {
             L2NUndoOp::NetlistOp(op) => self.undo_netlist_op(op),
             L2NUndoOp::LayoutOp(op) => self.undo_layout_op(op),
             // L2N specific operations
+            L2NUndoOp::SetNetOfShape {
+                shape_id,
+                previous_net
+            } => {
+                self.chip.set_net_of_shape(&shape_id, previous_net);
+            }
+            L2NUndoOp::SetPinOfShape {
+                shape_id,
+                previous_pin
+            } => {
+                self.chip.set_pin_of_shape(&shape_id, previous_pin);
+            }
         }
     }
 }
@@ -193,7 +258,7 @@ impl<'a, T: L2NEdit, U> Undo<'a, T, U> {
 impl<'a, T: L2NEdit> Undo<'a, T, L2NUndoOp<T>> {
     /// Create a wrapper around a fused layout and netlist which
     /// allows to undo operations.
-    pub fn undo_l2n(chip: &'a mut T) -> Self {
+    pub fn new_l2n_undo(chip: &'a mut T) -> Self {
         Self {
             chip,
             transactions: vec![],
@@ -220,13 +285,13 @@ impl<'a, T: LayoutEdit, U> Undo<'a, T, U> {
                 log::error!("Creating a layer cannot be undone.");
             }
             LayoutUndoOp::SetLayerName(id, old_name) =>
-                {self.chip.set_layer_name(&id, old_name);},
+                { self.chip.set_layer_name(&id, old_name); }
             LayoutUndoOp::InsertShape(id) =>
-                {self.chip.remove_shape(&id);},
+                { self.chip.remove_shape(&id); }
             LayoutUndoOp::RemoveShape { parent_cell, layer, geometry } => {
                 self.chip.insert_shape(&parent_cell, &layer, geometry);
             }
-            LayoutUndoOp::ReplaceShape (id, geometry) => {
+            LayoutUndoOp::ReplaceShape(id, geometry) => {
                 self.chip.replace_shape(&id, geometry);
             }
             LayoutUndoOp::SetTransform(inst, old_tf) => {
@@ -239,7 +304,7 @@ impl<'a, T: LayoutEdit, U> Undo<'a, T, U> {
 impl<'a, T: LayoutEdit> Undo<'a, T, LayoutUndoOp<T>> {
     /// Create a wrapper which allows to undo operations performed
     /// on the `LayoutEdit` trait.
-    pub fn undo_layout(chip: &'a mut T) -> Self {
+    pub fn new_layout_undo(chip: &'a mut T) -> Self {
         Self {
             chip,
             transactions: vec![],
@@ -280,7 +345,7 @@ impl<'a, T: NetlistEdit, U> Undo<'a, T, U> {
 impl<'a, T: NetlistEdit> Undo<'a, T, NetlistUndoOp<T>> {
     /// Create a wrapper which allows to undo operations performed
     /// on the `NetlistEdit` trait.
-    pub fn undo_netlist(chip: &'a mut T) -> Self {
+    pub fn new_netlist_undo(chip: &'a mut T) -> Self {
         Self {
             chip,
             transactions: vec![],
@@ -299,7 +364,7 @@ impl<'a, T: NetlistEdit> Undo<'a, T, NetlistUndoOp<T>> {
 impl<'a, T: HierarchyEdit> Undo<'a, T, HierarchyUndoOp<T>> {
     /// Create a wrapper which allows to undo operations performed
     /// on the `HierarchyEdit` trait.
-    pub fn undo_hierarchy(chip: &'a mut T) -> Self {
+    pub fn new_hierarchy_undo(chip: &'a mut T) -> Self {
         Self {
             chip,
             transactions: vec![],
@@ -323,72 +388,33 @@ impl<'a, T: HierarchyEdit> Undo<'a, T, HierarchyUndoOp<T>> {
     }
 }
 
-// impl<'a, T: HierarchyBase, U> HierarchyDelegate<T> for Undo<'a, T, U> {
-//     fn base(&self) -> &T {
-//         &self.chip
-//     }
-// }
 
-impl<'a, T: HierarchyBase, U> HierarchyBase for Undo<'a, T, U> {
-    // This is nothing but simple redirection.
-    type NameType = T::NameType;
-    type CellId = T::CellId;
-    type CellInstId = T::CellInstId;
+impl<'a, H, U> Decorator for Undo<'a, H, U> {
+    type D = H;
 
-    fn cell_by_name(&self, name: &str) -> Option<Self::CellId> {
-        self.chip.cell_by_name(name)
-    }
-
-    fn cell_instance_by_name(&self, parent_cell: &Self::CellId, name: &str) -> Option<Self::CellInstId> {
-        self.chip.cell_instance_by_name(parent_cell, name)
-    }
-
-    fn cell_name(&self, cell: &Self::CellId) -> Self::NameType {
-        self.chip.cell_name(cell)
-    }
-
-    fn cell_instance_name(&self, cell_inst: &Self::CellInstId) -> Option<Self::NameType> {
-        self.chip.cell_instance_name(cell_inst)
-    }
-
-    fn parent_cell(&self, cell_instance: &Self::CellInstId) -> Self::CellId {
-        self.chip.parent_cell(cell_instance)
-    }
-
-    fn template_cell(&self, cell_instance: &Self::CellInstId) -> Self::CellId {
-        self.chip.template_cell(cell_instance)
-    }
-
-    fn for_each_cell<F>(&self, f: F) where F: FnMut(Self::CellId) -> () {
-        self.chip.for_each_cell(f)
-    }
-
-    fn for_each_cell_instance<F>(&self, cell: &Self::CellId, f: F) where F: FnMut(Self::CellInstId) -> () {
-        self.chip.for_each_cell_instance(cell, f)
-    }
-
-    fn for_each_cell_dependency<F>(&self, cell: &Self::CellId, f: F) where F: FnMut(Self::CellId) -> () {
-        self.chip.for_each_cell_dependency(cell, f)
-    }
-
-    fn for_each_dependent_cell<F>(&self, cell: &Self::CellId, f: F) where F: FnMut(Self::CellId) -> () {
-        self.chip.for_each_dependent_cell(cell, f)
-    }
-
-    fn for_each_cell_reference<F>(&self, cell: &Self::CellId, f: F) where F: FnMut(Self::CellInstId) -> () {
-        self.chip.for_each_cell_reference(cell, f)
-    }
-
-    fn num_child_instances(&self, cell: &Self::CellId) -> usize {
-        self.chip.num_child_instances(cell)
-    }
-
-    fn num_cells(&self) -> usize {
-        self.chip.num_cells()
+    fn base(&self) -> &Self::D {
+        &self.chip
     }
 }
 
-impl<'a, T: HierarchyEdit, U: From<HierarchyUndoOp<T>>> HierarchyEdit for Undo<'a, T, U> {
+impl<'a, H, U> MutDecorator for Undo<'a, H, U> {
+    fn mut_base(&mut self) -> &mut Self::D {
+        &mut self.chip
+    }
+}
+
+// Inherit everything from HierarchyBase.
+impl<'a, H: HierarchyBase + 'static, U> HierarchyBaseDecorator for Undo<'a, H, U> {
+    type NameType = H::NameType;
+    type CellId = H::CellId;
+    type CellInstId = H::CellInstId;
+}
+
+// Inherit everything from LayoutBase.
+impl<'a, L: LayoutBase + 'static, U> LayoutBaseDecorator for Undo<'a, L, U> {}
+
+
+impl<'a, T: HierarchyEdit + 'static, U: From<HierarchyUndoOp<T>>> HierarchyEdit for Undo<'a, T, U> {
     fn new() -> Self {
         unimplemented!()
     }
@@ -414,19 +440,19 @@ impl<'a, T: HierarchyEdit, U: From<HierarchyUndoOp<T>>> HierarchyEdit for Undo<'
     }
 
     fn rename_cell_instance(&mut self, inst: &Self::CellInstId, new_name: Option<Self::NameType>) {
-        let prev_name = self.cell_instance_name(inst);
+        let previous_name = self.d_cell_instance_name(inst);
         self.chip.rename_cell_instance(inst, new_name);
-        self.transactions.push(HierarchyUndoOp::RenameCellInst(inst.clone(), prev_name).into());
+        self.transactions.push(HierarchyUndoOp::RenameCellInst { inst: inst.clone(), previous_name }.into());
     }
 
     fn rename_cell(&mut self, cell: &Self::CellId, new_name: Self::NameType) {
-        let prev_name = self.cell_name(cell);
+        let previous_name = self.d_cell_name(cell);
         self.chip.rename_cell(cell, new_name);
-        self.transactions.push(HierarchyUndoOp::RenameCell(cell.clone(), prev_name).into());
+        self.transactions.push(HierarchyUndoOp::RenameCell { cell: cell.clone(), previous_name }.into());
     }
 }
 
-impl<'a, T: NetlistBase, U> NetlistBase for Undo<'a, T, U> {
+impl<'a, T: NetlistBase + 'static, U> NetlistBase for Undo<'a, T, U> {
     type PinId = T::PinId;
     type PinInstId = T::PinInstId;
     type NetId = T::NetId;
@@ -509,7 +535,7 @@ impl<'a, T: NetlistBase, U> NetlistBase for Undo<'a, T, U> {
 }
 
 impl<'a, T, U> NetlistEdit for Undo<'a, T, U>
-    where T: NetlistEdit,
+    where T: NetlistEdit + 'static,
           U: From<NetlistUndoOp<T>> + From<HierarchyUndoOp<T>> {
     fn create_pin(&mut self, circuit: &Self::CellId, name: Self::NameType, direction: Direction) -> Self::PinId {
         let id = self.chip.create_pin(circuit, name, direction);
@@ -560,62 +586,8 @@ impl<'a, T, U> NetlistEdit for Undo<'a, T, U>
     }
 }
 
-
-impl<'a, T: LayoutBase, U> LayoutBase for Undo<'a, T, U> {
-
-    // Pass-through all functions of the LayoutBase trait.
-
-    type Coord = T::Coord;
-    type LayerId = T::LayerId;
-    type ShapeId = T::ShapeId;
-
-    fn dbu(&self) -> Self::Coord {
-        self.chip.dbu()
-    }
-
-    fn each_layer(&self) -> Box<dyn Iterator<Item=Self::LayerId> + '_> {
-        self.chip.each_layer()
-    }
-
-    fn layer_info(&self, layer: &Self::LayerId) -> &LayerInfo<Self::NameType> {
-        self.chip.layer_info(layer)
-    }
-
-    fn find_layer(&self, index: u32, datatype: u32) -> Option<Self::LayerId> {
-        self.chip.find_layer(index, datatype)
-    }
-
-    fn layer_by_name(&self, name: &str) -> Option<Self::LayerId> {
-        self.chip.layer_by_name(name)
-    }
-
-    fn bounding_box_per_layer(&self, cell: &Self::CellId, layer: &Self::LayerId) -> Option<Rect<Self::Coord>> {
-        self.chip.bounding_box_per_layer(cell, layer)
-    }
-
-    fn each_shape_id(&self, cell: &Self::CellId, layer: &Self::LayerId) -> Box<dyn Iterator<Item=Self::ShapeId> + '_> {
-        self.chip.each_shape_id(cell, layer)
-    }
-
-    fn for_each_shape<F>(&self, cell: &Self::CellId, layer: &Self::LayerId, f: F) where F: FnMut(&Self::ShapeId, &Geometry<Self::Coord>) -> () {
-        self.chip.for_each_shape(cell, layer, f)
-    }
-
-    fn with_shape<F, R>(&self, shape_id: &Self::ShapeId, f: F) -> R where F: FnMut(&Self::LayerId, &Geometry<Self::Coord>) -> R {
-        self.chip.with_shape(shape_id, f)
-    }
-
-    fn parent_of_shape(&self, shape_id: &Self::ShapeId) -> (Self::CellId, Self::LayerId) {
-        self.chip.parent_of_shape(shape_id)
-    }
-
-    fn get_transform(&self, cell_inst: &Self::CellInstId) -> SimpleTransform<Self::Coord> {
-        self.chip.get_transform(cell_inst)
-    }
-}
-
 impl<'a, T, U> LayoutEdit for Undo<'a, T, U>
-    where T: LayoutEdit,
+    where T: LayoutEdit + 'static,
           U: From<LayoutUndoOp<T>> + From<HierarchyUndoOp<T>> {
     fn set_dbu(&mut self, dbu: Self::Coord) {
         self.transactions.push(LayoutUndoOp::SetDbu(self.dbu()).into());
@@ -653,7 +625,7 @@ impl<'a, T, U> LayoutEdit for Undo<'a, T, U>
         geometry
     }
 
-    fn replace_shape(&mut self,  shape_id: &Self::ShapeId, geometry: Geometry<Self::Coord>) -> Geometry<Self::Coord> {
+    fn replace_shape(&mut self, shape_id: &Self::ShapeId, geometry: Geometry<Self::Coord>) -> Geometry<Self::Coord> {
         let old_geometry = self.chip.replace_shape(shape_id, geometry);
 
         self.transactions.push(LayoutUndoOp::ReplaceShape(shape_id.clone(), old_geometry.clone()).into());
@@ -673,12 +645,28 @@ impl<'a, T, U> LayoutEdit for Undo<'a, T, U>
     }
 }
 
+impl<'a, T, U> L2NEdit for Undo<'a, T, U>
+    where T: L2NEdit + 'static,
+          U: From<L2NUndoOp<T>> + From<LayoutUndoOp<T>> + From<NetlistUndoOp<T>> + From<HierarchyUndoOp<T>> {
+    fn set_pin_of_shape(&mut self, shape_id: &Self::ShapeId, pin: Option<Self::PinId>) -> Option<Self::PinId> {
+        let previous_pin = self.get_pin_of_shape(shape_id);
+        self.transactions.push(L2NUndoOp::SetPinOfShape { shape_id: shape_id.clone(), previous_pin }.into());
+        self.chip.set_pin_of_shape(shape_id, pin)
+    }
+
+    fn set_net_of_shape(&mut self, shape_id: &Self::ShapeId, net: Option<Self::NetId>) -> Option<Self::NetId> {
+        let previous_net = self.get_net_of_shape(shape_id);
+        self.transactions.push(L2NUndoOp::SetNetOfShape { shape_id: shape_id.clone(), previous_net }.into());
+        self.chip.set_net_of_shape(shape_id, net)
+    }
+}
+
 
 #[test]
 fn test_hierarchy_undoing() {
     use crate::chip::Chip;
     let mut chip = Chip::new();
-    let mut undo = Undo::undo_netlist(&mut chip);
+    let mut undo = Undo::new_netlist_undo(&mut chip);
 
     let top = undo.create_cell("TOP".into());
     let _top_a = undo.create_pin(&top, "A".into(), Direction::Input);
